package com.gxzc.zen.api.shared.service

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.mapper.Wrapper
import com.gxzc.zen.api.shared.model.ShareTime
import com.gxzc.zen.common.base.BaseModel
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.util.PlatformUtil
import com.gxzc.zen.orm.annotation.ZenTransactional
import com.xxl.job.core.log.XxlJobLogger
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import java.util.*

/**
 * 公共数据流转 抽象类<br>
 * 单表业务(单表作为主要流转对象的场景)
 * @author NorthLan
 * @date 2018/4/14
 * @url https://noahlan.com
 */
@Service
abstract class AbstractSharedService<T : BaseModel, K : BaseModel> {
    @Autowired
    private lateinit var shareTimeService: IShareTimeService

    /**
     * 获取最新共享时间
     */
    protected fun getLastTime(key: String): ShareTime {
        val conditionEntity = ShareTime().apply {
            this.platformId = PlatformUtil.getPlatformId()
            this.key = key
        }
        var lastTime: ShareTime? = shareTimeService.selectOne(EntityWrapper<ShareTime>(conditionEntity))
        if (lastTime == null) {
            if (shareTimeService.insert(conditionEntity)) {
                lastTime = conditionEntity
            } else {
                val e = ZenException(999, "公共表 [share_time] 数据插入失败!")
                XxlJobLogger.log(e)
                throw e
            }
        }
        if (lastTime.time == null) {
            lastTime.time = Date(0L)
        }
        return lastTime
    }

    /**
     * 从数据中获取最近的时间
     * 由于条件构造为: 按时间进行降序排列
     * 故 直接取第一条数据即可
     */
    private fun <A : BaseModel> getLastTimeFromDataList(data: List<A>): Date? {
        return if (data.isEmpty()) {
            null
        } else {
            data[0].updateTime ?: data[0].createTime
        }
    }

    /**
     * 更新共享时间
     */
    private fun updateShareTime(lastTime: ShareTime) {
        shareTimeService.updateById(lastTime)
    }

    /**
     * 构造时间查询条件
     * 结果 降序排列
     */
    protected fun <A : BaseModel> queryConditionWrapper(time: Date): Wrapper<A> {
        return EntityWrapper<A>()
                .where("(update_time is null and create_time > {0}) or (update_time is not null and update_time > {1})", time, time)
                .orderBy("create_time", false)
                .orderBy("update_time", false)
    }

    /**
     * 插入数据逻辑
     * (定时)
     */
    @ZenTransactional
    fun insert() {
        val lastTime = getLastTimeInsert()

        val result = innerInsert(lastTime)

        if (result != null) {
            val last = getLastTimeFromDataList(result)
            if (last != null) {
                updateShareTime(lastTime.apply { time = last })
            }
        }
    }

    abstract fun getLastTimeInsert(): ShareTime

    abstract fun innerInsert(lastTime: ShareTime): MutableList<T>?


    /**
     * 读取数据逻辑
     * (定时)
     */
    fun select() {
        val lastTime = getLastTimeSelect()

        val result = innerSelect(lastTime)

        if (result != null) {
            val last = getLastTimeFromDataList(result)
            if (last != null) {
                updateShareTime(lastTime.apply { time = last })
            }
        }
    }

    abstract fun getLastTimeSelect(): ShareTime

    abstract fun innerSelect(lastTime: ShareTime): MutableList<K>?

    /**
     * 公共字段 克隆
     */
    protected fun commonFieldClone(from: BaseModel, to: BaseModel) {
        to.apply {
            this.updateBy = from.updateBy
            this.createBy = from.createBy
            this.updateTime = from.updateTime
            this.createTime = from.createTime
            this.remark = from.remark
            this.enable = from.enable
//            this.id = from.id
        }
    }
}