package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysUserMapper
import com.gxzc.zen.api.sys.model.SysUser
import com.gxzc.zen.api.sys.service.ISysUserRoleService
import com.gxzc.zen.api.sys.service.ISysUserService
import com.gxzc.zen.common.contants.CACHEKEYS
import com.gxzc.zen.common.contants.PLATFORM
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.properties.PlatformProperties
import com.gxzc.zen.common.util.PlatformUtil
import com.gxzc.zen.common.util.RedisCacheUtil
import com.gxzc.zen.orm.annotation.ZenTransactional
import com.gxzc.zen.umps.util.MD5Salt
import org.apache.commons.lang3.RandomStringUtils
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.CommandLineRunner
import org.springframework.stereotype.Service

/**
 * <p>
 * 用户管理 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Suppress("UNCHECKED_CAST")
@Service
class SysUserServiceImpl : ServiceImpl<SysUserMapper, SysUser>(), ISysUserService, CommandLineRunner {
    companion object {
        private val logger = LoggerFactory.getLogger(SysUserServiceImpl::class.java)
        const val CACHE_KEY_ALL = "all"
    }

    @Autowired
    private lateinit var platformProperties: PlatformProperties

    @Autowired
    private lateinit var userRoleService: ISysUserRoleService

    override fun run(vararg args: String?) {
        logger.debug("${this::class.simpleName} init.")
        getListCacheable()
    }

    override fun getListCacheable(): MutableList<SysUser> {
        val cached = RedisCacheUtil.get(CACHEKEYS.USER, CACHE_KEY_ALL)?.get() as? MutableList<SysUser>
        if (cached != null) {
            return cached
        }
//        val ret = baseMapper.selectByParams(null)
        val ret = baseMapper.selectWOLogic(null)
        if (PlatformUtil.getPlatform(platformProperties) == PLATFORM.SYSTEM) {
            RedisCacheUtil.put(CACHEKEYS.USER, CACHE_KEY_ALL, ret)
        }
        return ret
    }

    override fun getUserByAccountCacheable(account: String): SysUser? {
        return getListCacheable().find {
            it.account == account
        }
    }

    override fun getUserByIdCacheable(id: Long): SysUser? {
        return getListCacheable().find {
            it.id == id
        }
    }

    @ZenTransactional
    override fun insertCacheable(entity: SysUser) {
        if (PlatformUtil.getPlatform() == PLATFORM.SYSTEM) {
            // 处理一下密码
            if (entity.password.isNullOrEmpty()) {
                throw ZenException(ZenExceptionEnum.REG_PASSWORD_ERROR)
            } else if (entity.password!!.length < 6) {
                throw ZenException(ZenExceptionEnum.REG_PASSWORD_ERROR)
            }

            val salt = RandomStringUtils.randomAlphanumeric(9)!!
            entity.salt = salt
            entity.password = MD5Salt.md5SaltEncode(salt, entity.password!!)

            if (baseMapper.insert(entity) == 0) {
                throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
            }

            // role
            val newRoleIdList = entity.roles?.map { it.id!! }
            if (newRoleIdList != null) {
                userRoleService.insertBatch(entity.id!!, newRoleIdList)
            }

            val cached = RedisCacheUtil.get(CACHEKEYS.USER, CACHE_KEY_ALL)?.get() as? MutableList<SysUser>
            cached?.let {
                it.add(entity)
                RedisCacheUtil.put(CACHEKEYS.USER, CACHE_KEY_ALL, it)
            }
        }
    }

    @ZenTransactional
    override fun modify(entity: SysUser): SysUser {
        if (PlatformUtil.getPlatform() == PLATFORM.SYSTEM) {
            /* 更新
             * 查询原有角色
             * 删除 / 新增 角色
             */
            val userRoleIdList = userRoleService.getUserRoleListByUserId(entity.id!!).map { it.id!! }
            val newRoleIdList = entity.roles?.map { it.id!! }
            if (newRoleIdList != null) {
                // del
                val delRoleIdList = userRoleIdList.toMutableList()
                delRoleIdList.removeAll(newRoleIdList)
                // add
                val addRoleIdList = newRoleIdList.toMutableList()
                addRoleIdList.removeAll(userRoleIdList)

                userRoleService.physicalDeleteBatch(entity.id!!, delRoleIdList)
                userRoleService.insertBatch(entity.id!!, addRoleIdList)
            }

            // 密码搞一下
            if (!entity.password.isNullOrEmpty() && entity.password!!.length >= 6) {
                val salt = RandomStringUtils.randomAlphanumeric(9)!!
                entity.salt = salt
                entity.password = MD5Salt.md5SaltEncode(salt, entity.password!!)
            }

            baseMapper.updateWOLogic(entity, EntityWrapper<SysUser>().eq("id", entity.id))
//            throw ZenException(ZenExceptionEnum.SERVER_ERROR)
            // 更新缓存
            val cached = RedisCacheUtil.get(CACHEKEYS.USER, CACHE_KEY_ALL)?.get() as? MutableList<SysUser>
            cached?.let {
                val idx = it.indexOfFirst { it.id == entity.id }
                if (idx != -1) {
                    it[idx] = entity
                }
                RedisCacheUtil.put(CACHEKEYS.USER, CACHE_KEY_ALL, it)
            }
        }
        return entity
    }

    @ZenTransactional
    override fun physicalDeleteCacheable(id: Long) {
        if (PlatformUtil.getPlatform() == PLATFORM.SYSTEM) {
            if (baseMapper.physicalDelete(EntityWrapper<SysUser>().eq("id", id)) <= 0) {
                throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
            }
            // 删除与之关联的所有系统表数据
            userRoleService.physicalDeleteByUserId(id)
            //
            val cached = RedisCacheUtil.get(CACHEKEYS.USER, CACHE_KEY_ALL)?.get() as? MutableList<SysUser>
            cached?.let {
                it.removeIf {
                    it.id == id
                }
                RedisCacheUtil.put(CACHEKEYS.USER, CACHE_KEY_ALL, it)
            }
        }
    }
}
