package com.gxzc.zen.orm.config

import com.baomidou.mybatisplus.MybatisConfiguration
import com.baomidou.mybatisplus.MybatisXMLLanguageDriver
import com.baomidou.mybatisplus.spring.MybatisSqlSessionFactoryBean
import com.baomidou.mybatisplus.spring.boot.starter.ConfigurationCustomizer
import com.baomidou.mybatisplus.spring.boot.starter.MybatisPlusProperties
import com.baomidou.mybatisplus.spring.boot.starter.SpringBootVFS
import com.gxzc.zen.orm.DynamicMultipleDataSource
import com.gxzc.zen.orm.contants.DSKey
import org.apache.ibatis.mapping.DatabaseIdProvider
import org.apache.ibatis.plugin.Interceptor
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.ObjectProvider
import org.springframework.beans.factory.annotation.Qualifier
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.Primary
import org.springframework.core.io.ResourceLoader
import org.springframework.util.Assert
import org.springframework.util.CollectionUtils
import org.springframework.util.ObjectUtils
import org.springframework.util.StringUtils
import javax.annotation.PostConstruct
import javax.sql.DataSource

/**
 * Mybatis bean工厂配置
 * @author NorthLan
 * @date 2018/3/27
 * @url https://noahlan.com
 */
@Configuration
@EnableConfigurationProperties(MybatisPlusProperties::class)
class MybatisPlusFactoryConfig {
    companion object {
        private val logger = LoggerFactory.getLogger(MybatisPlusFactoryConfig::class.java)
    }

    private var interceptors: Array<Interceptor>? = null
    private var properties: MybatisPlusProperties? = null
    private var resourceLoader: ResourceLoader? = null
    private var databaseIdProvider: DatabaseIdProvider? = null
    private var configurationCustomizers: List<ConfigurationCustomizer>? = null

    init {
        logger.info("MybatisPlusFactoryConfig initializing...")
    }

    constructor(properties: MybatisPlusProperties,
                interceptorsProvider: ObjectProvider<Array<Interceptor>>,
                resourceLoader: ResourceLoader,
                databaseIdProvider: ObjectProvider<DatabaseIdProvider>,
                configurationCustomizersProvider: ObjectProvider<List<ConfigurationCustomizer>>) {
        this.properties = properties
        this.interceptors = interceptorsProvider.ifAvailable
        this.resourceLoader = resourceLoader
        this.databaseIdProvider = databaseIdProvider.ifAvailable
        this.configurationCustomizers = configurationCustomizersProvider.ifAvailable
    }

    @PostConstruct
    fun checkConfigFileExists() {
        Assert.state(properties != null, "MybatisPlusProperties is null.")
        if (this.properties!!.isCheckConfigLocation && StringUtils.hasText(this.properties?.configLocation)) {
            val resource = this.resourceLoader?.getResource(this.properties?.configLocation)
            Assert.state(resource != null && resource.exists(), "Cannot find config location: " + resource
                    + " (please add config file or check your Mybatis configuration)")
        }
    }

    @Bean
    @Primary
    @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
    fun dynamic(dataSource: DynamicMultipleDataSource): MybatisSqlSessionFactoryBean {
        return setFactoryBean(dataSource)
    }

    @Bean
    @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "false", matchIfMissing = true)
    fun single(@Qualifier(DSKey.DSKEY_SYS) dataSource: DataSource): MybatisSqlSessionFactoryBean {
        return setFactoryBean(dataSource)
    }

    private fun setFactoryBean(dataSource: DataSource/*, mapperLocation: String?*/): MybatisSqlSessionFactoryBean {
        return MybatisSqlSessionFactoryBean().apply {
            setDataSource(dataSource)
            vfs = SpringBootVFS::class.java
            if (StringUtils.hasText(properties?.configLocation)) {
                setConfigLocation(resourceLoader?.getResource(properties?.configLocation))
            }
            var configuration = properties?.configuration
            if (configuration == null && !StringUtils.hasText(properties?.configLocation)) {
                configuration = MybatisConfiguration()
            }
            if (configuration != null && !CollectionUtils.isEmpty(configurationCustomizers)) {
                configurationCustomizers?.forEach {
                    it.customize(configuration)
                }
            }
            configuration?.setDefaultScriptingLanguage(MybatisXMLLanguageDriver::class.java)
            setConfiguration(properties?.configuration)
            if (properties?.configurationProperties != null) {
                setConfigurationProperties(properties?.configurationProperties)
            }
            if (!ObjectUtils.isEmpty(interceptors)) {
                setPlugins(interceptors)
            }
            if (databaseIdProvider != null) {
                this.databaseIdProvider = databaseIdProvider
            }
            if (StringUtils.hasLength(properties?.typeAliasesPackage)) {
                setTypeAliasesPackage(properties?.typeAliasesPackage)
            }
            // 自定义枚举
            if (StringUtils.hasLength(properties?.typeEnumsPackage)) {
                setTypeEnumsPackage(properties?.typeEnumsPackage)
            }
            if (StringUtils.hasLength(properties?.typeHandlersPackage)) {
                setTypeHandlersPackage(properties?.typeHandlersPackage)
            }
            if (!ObjectUtils.isEmpty(properties?.resolveMapperLocations())) {
                setMapperLocations(properties?.resolveMapperLocations())
            }
//            if (StringUtils.hasLength(mapperLocation)) {
//                setMapperLocations(PathMatchingResourcePatternResolver().getResources(mapperLocation))
//            }

            if (!ObjectUtils.isEmpty(properties?.globalConfig)) {
                setGlobalConfig(properties?.globalConfig?.convertGlobalConfiguration())
            }
        }
    }

}