package com.gxzc.zen.web.sys.controller

import com.gxzc.zen.api.sys.model.SysRole
import com.gxzc.zen.api.sys.model.SysUser
import com.gxzc.zen.api.sys.service.ISysUserRoleService
import com.gxzc.zen.api.sys.service.ISysUserService
import com.gxzc.zen.common.base.BaseController
import com.gxzc.zen.common.config.response.annotation.ZenResponseFilter
import com.gxzc.zen.common.config.response.annotation.ZenResponseFilters
import com.gxzc.zen.common.dto.RequestDto
import com.gxzc.zen.common.dto.ResponseDto
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.util.PaginationUtil
import com.gxzc.zen.umps.util.SSOUtil
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.net.URI

/**
 * 用户相关控制器
 * @author NorthLan
 * @date 2018/3/2
 * @url https://noahlan.com
 */
@RestController
@RequestMapping("user")
class UserController : BaseController() {
    companion object {
        private val logger = LoggerFactory.getLogger(UserController::class.java)
    }

    @Autowired
    private lateinit var userRoleService: ISysUserRoleService

    @Autowired
    private lateinit var userService: ISysUserService


    @GetMapping("{id}")
    @ZenResponseFilter(type = SysUser::class, filter = ["createTime", "createBy", "updateTime", "updateBy", "password", "salt"])
    fun getById(@PathVariable id: Long): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto().apply {
            data = userService.selectById(id)
        })
    }

    @PutMapping
    @ZenResponseFilter(type = SysUser::class, filter = ["createTime", "createBy", "updateTime", "updateBy", "password", "salt"])
    fun putUser(@RequestBody data: SysUser): ResponseEntity<*> {
        return if (data.id != null) {
            ResponseEntity.ok(ResponseDto().apply {
                this.data = userService.modify(data) // 200
            })
        } else {
            // 插入,新增 角色
            userService.insertCacheable(data)
            ResponseEntity.created(URI.create("/user/${data.id}")).body(ResponseDto()) // 201
        }
    }

    @DeleteMapping("{id}")
    fun delete(@PathVariable id: Long): ResponseEntity<*> {
        userService.physicalDeleteCacheable(id)
        return ResponseEntity.ok(ResponseDto())
    }

    @GetMapping("/list")
    @ZenResponseFilters(
            ZenResponseFilter(type = SysUser::class, filter = ["createTime", "createBy", "updateTime", "updateBy", "password", "salt"]),
            ZenResponseFilter(type = SysRole::class, include = ["id", "name"])
    )
    fun list(@RequestParam(required = false) keyword: String?,
             @RequestParam(required = false) searchOption: Int?): ResponseEntity<*> {
        // 获取用户列表(忽略enable)
        var data: MutableList<SysUser> = userService.getListCacheable()
        if (!keyword.isNullOrEmpty() && searchOption != null) {
            data = data.filter {
                when (searchOption) {
                    1 -> run {
                        if (it.account != null) {
                            keyword!! in it.account!!
                        } else {
                            false
                        }
                    }
                    2 -> run {
                        if (it.username != null) {
                            keyword!! in it.username!!
                        } else {
                            false
                        }
                    }
                    3 -> run {
                        if (it.staffNo != null) {
                            keyword!! in it.staffNo!!
                        } else {
                            false
                        }
                    }
                    else -> false
                }
            }.toMutableList()
        }

        // 查出用户角色列表
        data.forEach {
            if (it.id != null) {
                it.roles = userRoleService.getUserRoleListByUserId(it.id!!)
            }
        }

        return if (PaginationUtil.paginable(getRequest())) {
            // 分页
            ResponseEntity.ok(ResponseDto().apply {
                this.data = PaginationUtil.logicPaging(data, getRequest())
            })
        } else {
            ResponseEntity.ok(ResponseDto().apply {
                this.data = data
            })
        }
    }


    @GetMapping("/info")
    @ZenResponseFilters(
            ZenResponseFilter(type = SysUser::class, include = ["id", "enable", "account", "username", "position", "address", "staffNo", "remark"]),
            ZenResponseFilter(type = SysRole::class, include = ["id", "enable", "name", "code", "perms", "remark"])
    )
    fun userInfo(): ResponseEntity<*> {
        val uid = SSOUtil.getCurUserId() ?: throw ZenException(ZenExceptionEnum.AUTH_NO_LOGIN)
        return ResponseEntity.ok(ResponseDto().apply {
            data = mutableMapOf(
                    "info" to userService.getUserByIdCacheable(uid),
                    "roles" to userRoleService.getUserRoleListByUserId(uid)
            )
        })
    }

    @PutMapping("/info")
    fun modifyUserInfo(@RequestBody requestDto: RequestDto): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto())
    }
}